@@ -1,16 +1,23 @@
 # Schedule::Load::Hosts.pm -- Loading information about hosts
-# $Id: Hosts.pm,v 1.73 2005/10/13 12:15:30 wsnyder Exp $
+# $Id: Hosts.pm,v 1.37 2002/08/01 14:46:03 wsnyder Exp $
 ######################################################################
 #
-# Copyright 2000-2004 by Wilson Snyder.  This program is free software;
-# you can redistribute it and/or modify it under the terms of either the GNU
-# General Public License or the Perl Artistic License.
+# This program is Copyright 2002 by Wilson Snyder.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of either the GNU General Public License or the
+# Perl Artistic License, with the exception that it cannot be placed
+# on a CD-ROM or similar media for commercial distribution without the
+# prior approval of the author.
 # 
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 # 
+# If you do not have a copy of the GNU General Public License write to
+# the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
+# MA 02139, USA.
 ######################################################################
 
 package Schedule::Load::Hosts;
@@ -19,9 +26,8 @@ require Exporter;
 @ISA = qw(Exporter);
 
 use Socket;
-use POSIX qw (EWOULDBLOCK EINTR EAGAIN BUFSIZ);
+
 use Schedule::Load qw(:_utils);
-use Schedule::Load::Hold;
 use Schedule::Load::Hosts::Host;
 use Schedule::Load::Hosts::Proc;
 use Time::localtime;
@@ -37,7 +43,7 @@ use Carp;
 # Other configurable settings.
 $Debug = $Schedule::Load::Debug;
 
-$VERSION = '2.022';
+$VERSION = '2.090';
 
 ######################################################################
 #### Globals
@@ -68,7 +74,7 @@ sub fetch {
     # Erase current structures in case a host goes down
     delete $self->{hosts};
     # Make the request
-    $self->_request("get_const_load_proc_chooinfo\n");
+    $self->_request("get_const_load_proc\n");
     $self->{_fetched} = 1;
     return $self;
 }
@@ -102,7 +108,7 @@ sub _chooser_close_all {
 
 sub hosts {
     my $self = shift; ($self && ref($self)) or croak 'usage: $self->hosts()';
-    # Return all hosts
+    # Return all hosts, potentially matching given classes
 
     $self->_fetch_if_unfetched;
     my @keys;
@@ -113,32 +119,6 @@ sub hosts {
     return (wantarray ? @keys : \@keys);
 }
 
-sub hosts_match {
-    my $self = shift; ($self && ref($self)) or croak 'usage: $self->hosts_match()';
-    my %params = (#classes=>[],		# Passed to Host::host_match
-		  #match_cb=>0,		# Passed to Host::host_match
-		  #allow_reserved=>1,	# Passed to Host::host_match
-		  @_);
-    # Return all hosts matching parameters
-    $self->_fetch_if_unfetched;
-    my @keys;
-    foreach my $host ($self->hosts) {
-	push @keys, $host if $host->host_match(%params);
-    }
-    return (wantarray ? @keys : \@keys);
-}
-
-sub schreq_holds {
-    my $self = shift; ($self && ref($self)) or croak 'usage: $self->schreqs_holds()';
-    # Return all hosts matching parameters
-    $self->_fetch_if_unfetched;
-    my @keys;
-    foreach my $hold (values(%{$self->{chooinfo}{schreqs}})) {
-	push @keys, $hold;
-    }
-    return (wantarray ? @keys : \@keys);
-}
-
 sub get_host {
     my $self = shift; ($self && ref($self)) or croak 'usage: $self->get_host(hostname)';
     my $hostname = shift;
@@ -168,59 +148,35 @@ sub classes {
 
 sub cpus {
     my $self = shift; ($self && ref($self)) or croak 'usage: $self->classes()';
-    my %params = (#classes=>[],		# Passed to Host::host_match
-		  #match_cb=>0,		# Passed to Host::host_match
-		  allow_reserved=>1,	# Passed to Host::host_match
-		  @_);
+    my %params = @_;
     # Return number of cpus for a given class
+
     $self->_fetch_if_unfetched;
     my $jobs = 0;
-    foreach my $host ($self->hosts_match(%params)) {
-	$jobs += $host->cpus();
+    foreach my $host ( @{$self->hosts} ){
+	if ($host->classes_match ($params{classes})) {
+	    $jobs += $host->cpus();
+	}
     }
     return $jobs;
 }
 
-sub hostnames {
-    my $self = shift; ($self && ref($self)) or croak 'usage: $self->hosts()';
-    my %params = (#classes=>[],		# Passed to Host::host_match
-		  #match_cb=>0,		# Passed to Host::host_match
-		  allow_reserved=>1,	# Passed to Host::host_match
-		  @_);
-    # Return hostnames, potentially matching given classes
-    my @hnames;
-    foreach my $host ($self->hosts_match(%params)) {
-	push @hnames, $host->hostname;
-    }
-    @hnames = (sort @hnames);
-    return (wantarray ? @hnames : \@hnames);
-}
-
 sub idle_host_names {
     my $self = shift; ($self && ref($self)) or croak 'usage: $self->hosts()';
-    my %params = (#classes=>[],		# Passed to Host::host_match
-		  #match_cb=>0,		# Passed to Host::host_match
-		  allow_reserved=>0,	# Passed to Host::host_match
-		  #ign_pctcpu=>0,
-		  #by_pctcpu=>0,
-		  @_);
+    my %params = @_;
     # Return idle hosts, potentially matching given classes
     # Roughly scaled so even powered hosts have even representation
 
+    $self->_fetch_if_unfetched;
     my @hnames;
-    foreach my $host ($self->hosts_match(%params)) {
-	my $idleCpus = $host->cpus;
-	if ($params{ign_pctcpu}) {
-	} elsif ($params{by_pctcpu}) {  # min of adj_load or percentage
-	    $idleCpus = $host->cpus;
-	    my $adj = (($host->cpus * $host->total_pctcpu / 100) - 0.2);  # 80% used? squeeze another in
-	    $adj = 0 if $adj<0;
-	    $idleCpus -= $adj;
-	} else {
-	    $idleCpus = $host->free_cpus;
-	}
-	for (my $c=0; $c<$idleCpus; $c++) {
-	    push @hnames, $host->hostname;
+    foreach my $host (values %{$self->{hosts}}) {
+	if ($host->exists('hostname') && $host->hostname
+	    && $host->classes_match ($params{classes})
+	    && !$host->reserved) {
+	    my $idleCpus = $host->cpus - $host->adj_load;
+	    for (my $c=0; $c<$idleCpus; $c++) {
+		push @hnames, $host->hostname;
+	    }
 	}
     }
     @hnames = (sort @hnames);
@@ -245,18 +201,22 @@ sub print_hosts {
     my $hosts = shift;
     # Overall machine status
     my $out = "";
-    (my $FORMAT =           "%-12s    %4s     %4s   %6s%%       %5s   %6s     %2s    %s\n") =~ s/\s\s+/ /g;
+    (my $FORMAT =           "%-12s    %4s     %4s   %6s%%       %5s   %5s     %2s    %s\n") =~ s/\s\s+/ /g;
     $out.=sprintf ($FORMAT, "HOST", "CPUs", "FREQ", "TotCPU", "LOAD", "RATE", "RL", "ARCH/OS");
     foreach my $host ( @{$hosts->hosts} ){
 	my $ostype = $host->archname ." ". $host->osvers;
+	foreach (sort ($host->fields)) {
+	    $ostype .= " $_" if (/^lab_/);
+	}
 	$ostype = "Reserved: ".$host->reserved if ($host->reserved);
 	$out.=sprintf ($FORMAT,
 		       $host->hostname, 
-		       $host->cpus_slash, 
+		       $host->cpus, 
 		       $host->max_clock, 
 		       sprintf("%3.1f", $host->total_pctcpu), 
-		       $host->adj_load,
-		       $host->rating_text,
+		       $host->adj_load, 
+		       ( ($host->rating && !$host->reserved)
+			 ?sprintf("%4.2f", $host->rating):"inf"), 
 		       ( ($host->reservable?"R":" ")
 			 . digit($host,'load_limit')),
 		       $ostype,
@@ -265,83 +225,6 @@ sub print_hosts {
     return $out;
 }
 
-sub print_holds {
-    my $hosts = shift;
-    # Holding commands
-    my %holdlist;
-    my $i=0;
-    foreach my $host ($hosts->hosts) {
-	foreach my $hold ($host->holds) {
-	    $i++;
-	    my $key = $hold->req_user."_".$hold->req_hostname."_".$hold->req_pid
-		."_".$hold->hold_key."_".$host->hostname."_".$i;
-	    $holdlist{$key} = {hold => $hold,
-			       host => $host,
-			       code => ($hold->allocated?"A":"S"),};
-	}
-    }
-    foreach my $hold ($hosts->schreq_holds) {
-	my $key = $hold->req_user."_".$hold->req_hostname."_".$hold->req_pid
-	    ."_".$hold->hold_key."_CHOO_".$i;
-	$holdlist{$key} = {hold => $hold,
-			   host => undef,
-		           code => "P",};
-    }
-    my $out = "";
-    (my $FORMAT =           "%-8s   %-12s    %5s      %5s   %2s  %1s   %7s    %-12s %-s\n") =~ s/\s\s+/ /g;
-    $out.=sprintf ($FORMAT, "USER", "UHOST", "UPID", "PRI", "L", "S", "WAIT", "ON_HOST", "COMMENT");
-    foreach my $key (sort (keys %holdlist)) {
-	my $hold = $holdlist{$key}{hold};
-	my $host = $holdlist{$key}{host};
-	$out.=sprintf ($FORMAT,
-		       $hold->req_user,
-		       $hold->req_hostname,
-		       $hold->req_pid,
-		       $hold->req_pri,
-		       $hold->hold_load,
-		       $holdlist{$key}{code},
-		       Schedule::Load::Hosts::Proc->format_hhmm(time() - $hold->req_time),
-		       #
-		       ($host ? $host->hostname : "{pending}"), 
-		       $hold->comment,
-		       );
-    }
-    return $out;
-}
-
-sub print_status {
-    my $hosts = shift;
-    # Daemon status, mostly for debugging
-    my $out = "";
-    {
-	(my $FORMAT =           "%-12s     %-17s        %s\n") =~ s/\s\s+/ /g;
-	$out.=sprintf ($FORMAT, "CHOOSER", "CONNECTED", "DAEMON STATUS");
-	my $t = localtime($hosts->{chooinfo}{slchoosed_connect_time}||0);
-	my $tm = sprintf("%04d/%02d/%02d %02d:%02d", $t->year+1900,$t->mon+1,$t->mday,$t->hour,$t->min);
-	$out.=sprintf ($FORMAT,
-		       $hosts->{chooinfo}{slchoosed_hostname},
-		       $tm,
-		       $hosts->{chooinfo}{slchoosed_status});
-    }
-
-    (my $FORMAT =           "%-12s  %6s%%     %5s     %6s    %-17s        %6s      %s\n") =~ s/\s\s+/ /g;
-    $out.=sprintf ($FORMAT, "HOST", "TotCPU","LOAD", "RATE", "CONNECTED", "DELAY", "DAEMON STATUS");
-    foreach my $host ( @{$hosts->hosts} ){
-	my $t = localtime($host->slreportd_connect_time||0);
-	my $tm = sprintf("%04d/%02d/%02d %02d:%02d", $t->year+1900,$t->mon+1,$t->mday,$t->hour,$t->min);
-	$out.=sprintf ($FORMAT,
-		       $host->hostname, 
-		       sprintf("%3.1f", $host->total_pctcpu), 
-		       $host->adj_load, 
-		       $host->rating_text,
-		       $tm,
-		       (defined $host->slreportd_delay ? sprintf("%2.3f",$host->slreportd_delay) : "?"),
-		       $host->slreportd_status,
-		       );
-    }
-    return $out;
-}
-
 sub print_top {
     my $hosts = shift;
     # Top processes
@@ -371,8 +254,8 @@ sub print_loads {
     my $hosts = shift;
     # Top processes
     my $out = "";
-    (my $FORMAT =           "%-12s   %6s    %-8s    %3s   %6s     %5s%%    %s\n") =~ s/\s\s+/ /g;
-    $out.=sprintf ($FORMAT, "HOST", "PID", "USER", "NIC", "RUNTM", "CPU","COMMAND"); 
+    (my $FORMAT =           "%-12s   %6s    %-8s    %6s     %5s%%    %s\n") =~ s/\s\s+/ /g;
+    $out.=sprintf ($FORMAT, "HOST", "PID", "USER", "RUNTM", "CPU","COMMAND"); 
     foreach my $host ( @{$hosts->hosts} ){
 	foreach my $p ( sort {$b->pctcpu <=> $a->pctcpu}
 			@{$host->top_processes} ) {
@@ -380,9 +263,7 @@ sub print_loads {
 	    $out.=sprintf ($FORMAT, 
 			   $host->hostname,
 			   $p->pid, 
-			   $p->uname,
-			   $p->nice,
-			   $p->time_hhmm,
+			   $p->uname, 		$p->time_hhmm,
 			   sprintf("%3.1f", $p->pctcpu),
 			   $comment,
 			   );
@@ -393,19 +274,15 @@ sub print_loads {
 
 sub print_kills {
     my $hosts = shift;
-    my $params = {
-	signal=>0,
-	@_,};
     # Top processes
     my $out = "";
-    (my $FORMAT =           "ssh %-12s kill %s%6s #   %-8s    %6s     %5s%%    %s\n") =~ s/\s\s+/ /g;
+    (my $FORMAT =           "ssh %-12s kill %6s #   %-8s    %6s     %5s%%    %s\n") =~ s/\s\s+/ /g;
     foreach my $host ( @{$hosts->hosts} ){
 	foreach my $p ( sort {$b->pctcpu <=> $a->pctcpu}
 			@{$host->top_processes} ) {
 	    my $comment = ($p->exists('cmndcomment')? $p->cmndcomment:$p->fname);
 	    $out.=sprintf ($FORMAT, 
 			   $host->hostname,
-			   ($params->{signal}?"-$params->{signal} ":""),
 			   $p->pid, 
 			   $p->uname, 		$p->time_hhmm,
 			   sprintf("%3.1f", $p->pctcpu),
@@ -423,51 +300,31 @@ sub print_classes {
 
     my @classes = (sort ($hosts->classes()));
     my $classnum = 0;
-    my %class_letter;
-    my %class_numeric;
-    my @col_width;
+    my %class_letter = ();
     foreach my $class (@classes) {
 	$class_letter{$class} = chr($classnum%26+ord("a"));
-	$col_width[$classnum] = 1;
-	foreach my $host ( @{$hosts->hosts} ){
-	    my $val = $host->get_undef($class);
-	    if ($val) {
-		$col_width[$classnum] = length $val if $col_width[$classnum] < length $val;
-		$class_numeric{$class} = 1 if $val>1;
-	    }
-	}
 	$classnum++;
     }
-
     my $classes = $classnum;
     $classnum = 0;
     foreach my $class (@classes) {
-	$out.=sprintf ("%-12s ", ($classnum==$classes-1)?"HOST":"");
-	for (my $prtclassnum = 0; $prtclassnum<$classnum; $prtclassnum++) {
-	    $out .= (" "x$col_width[$prtclassnum])."|";
-	}
-	$out .= (" "x$col_width[$classnum]).$class_letter{$class};
-	for (my $prtclassnum = $classnum+1; $prtclassnum<$#classes; $prtclassnum++) {
-	    $out .= ("-"x$col_width[$prtclassnum])."-";
-	}
-	$out.= "-$class_letter{$class}" if $classnum!=$classes-1;
-	$out.=sprintf ("- %s\n", $class);
+	$class_letter{$class} = chr($classnum%26+ord("a"));
+	$out.=sprintf ("%-12s  %s%s%s %s\n",
+		       ($classnum==$classes-1)?"HOST":"", 
+		       "| "x$classnum, 
+		       $class_letter{$class}, "--"x($classes-$classnum),
+		       $class);
 	$classnum++;
     }
     foreach my $host ( @{$hosts->hosts} ){
 	$out .= sprintf "%-12s ", $host->hostname;
 	$classnum = 0;
 	foreach my $class (@classes) {
-	    my $val = $host->get_undef($class);
-	    my $chr = ".";
-	    if ($val && ($val > 1 || $class_numeric{$class})) {
-		$chr = $val;
-	    } elsif ($val) {
-		$chr = $class_letter{$class};
+	    if ($host->exists($class) && $host->get($class)) {
+		$out .= sprintf (" %s", $class_letter{$class});
 	    } else {
-		$chr = ".";
+		$out .= sprintf (" .");
 	    }
-	    $out .= sprintf (" %$col_width[$classnum]s", $chr);
 	    $classnum++;
 	}
 	$out .= "\n";
@@ -524,7 +381,6 @@ sub _open {
 	    . "\tYou probably need to run slchoosed\n$self->_request(): Stopped";
     }
     $self->{_fh} = $fh;
-    $self->{_inbuffer} = "";
 }
 
 sub _request {
@@ -537,46 +393,27 @@ sub _request {
     my $fh = $self->{_fh};
     
     print "_request-> $cmd\n" if $Debug;
-    $fh->send_and_check($cmd);
-
-    my $done;
-    my $eof;
-    while (!$done) {
-	if ($self->{_inbuffer} !~ /\n/) {
-	    my $data = '';
-	    $!=undef;
-	    my $rv = $fh->sysread($data, POSIX::BUFSIZ, 0);
-	    $self->{_inbuffer} .= $data;
-	    $eof = 1 if (!defined $rv || (length $data == 0))
-		&& ($! != POSIX::EINTR && $! != POSIX::EAGAIN);
-	    $done ||= $eof;
-	}
-
-	while ($self->{_inbuffer} =~ s/^([^\n]*)\n//) {
-	    my $line = $1;
-	    chomp $line;
-	    print "GOT $line\n" if $Debug;
-	    my ($cmd, $params) = _pthaw($line, $Debug);
-	    next if $line =~ /^\s*$/;
-	    if ($cmd eq "DONE") {
-		$done = 1;
-	    } elsif ($cmd eq "host") {
-		$self->_host_load ($params);
-	    } elsif ($cmd eq "schrtn") {
-		$self->{_schrtn} = $params;
-	    } elsif ($cmd eq "chooinfo") {
-		$self->{chooinfo} = $params;
-	    } else {
-		warn "%Warning: Bad Schedule::Load server response: $line\n";
-		$line = undef;
-	    }
+    print $fh $cmd;
+    my $line;
+    while (defined ($line = <$fh>)) {
+	chomp $line;
+	#print "GOT $line\n" if $Debug;
+	my ($cmd, $params) = _pthaw($line, $Debug);
+	next if $line =~ /^\s*$/;
+	last if $cmd eq "DONE";
+	if ($cmd eq "host") {
+	    $self->_host_load ($params);
+	} elsif ($cmd eq "best") {
+	    $self->{_best} = $params;
+	} else {
+	    warn "%Warning: Bad Schedule::Load server response: $line\n";
+	    $line = undef;
 	}
     }
-    if ($eof || !$fh->connected()) {
+    if (!defined $line) {
 	$fh->close();
 	undef $self->{_fh};
     }
-    print "_request DONE-> $cmd\n" if $Debug;
 }
 
 ######################################################################
@@ -618,6 +455,7 @@ __END__
 
 =pod
 
+
 =head1 NAME
 
 Schedule::Load::Hosts - Return host loading information across a network
@@ -637,7 +475,7 @@ Schedule::Load::Hosts - Return host loading information across a network
     foreach my $host ($hosts->hosts) {
 	printf STDOUT ($FORMAT,
 		       $host->hostname, 
-		       $host->cpus_slash, 
+		       $host->cpus, 
 		       $host->max_clock, 
 		       sprintf("%3.1f", $host->total_pctcpu), 
 		       $host->adj_load, 
@@ -678,18 +516,10 @@ parameter if true (default) restarts reporter.
 
 =item hosts ()
 
-Returns the host objects, accessible with L<Schedule::Load::Hosts::Host>.
+Returns the host objects, accessable with C<Schedule::Load::Hosts::Host>.
 In an array context, returns a list; In a a scalar context, returns a
 reference to a list.
 
-=item hosts_match (...)
-
-Returns L<Schedule::Load::Hosts::Host> objects for every host that matches
-the specified criteria.  Criteria are named parameters, as described in
-Schedule::Load::Schedule, of the following: classes specifies an arrayref
-of allowed classes.  match_cb is a routine returning true if this host
-matches.  allow_reserved=>0 disables returning of reserved hosts.
-
 =item idle_host_names (...)
 
 Returns a list of host cpu names which are presently idle.  Multiple
@@ -723,7 +553,7 @@ format.
 =item print_top
 
 Returns a string with the top jobs on all machines in a printable format,
-ala the L<top> program.
+ala the C<top> program.
 
 =item print_loads
 
@@ -748,22 +578,18 @@ The port number of slchoosed.  Defaults to 'slchoosed' looked up via
 
 =back
 
+=head1 SEE ALSO
+
+C<Schedule::Load::Hosts::Host>, C<Schedule::Load::Hosts::Proc>
+
+C<Schedule::Load>, C<rschedule>
+
 =head1 DISTRIBUTION
 
-The latest version is available from CPAN and from L<http://www.veripool.com/>.
-
-Copyright 1998-2004 by Wilson Snyder.  This package is free software; you
-can redistribute it and/or modify it under the terms of either the GNU
-Lesser General Public License or the Perl Artistic License.
+The latest version is available from CPAN.
 
 =head1 AUTHORS
 
 Wilson Snyder <wsnyder@wsnyder.org>
 
-=head1 SEE ALSO
-
-L<Schedule::Load>, L<rschedule>
-
-L<Schedule::Load::Hosts::Host>, L<Schedule::Load::Hosts::Proc>
-
 =cut
