@@ -1,16 +1,23 @@
 # Schedule::Load::Hosts::Host.pm -- Loading information about a host
-# $Id: Host.pm,v 1.41 2005/10/13 12:15:30 wsnyder Exp $
+# $Id: Host.pm,v 1.16 2002/08/01 14:46:03 wsnyder Exp $
 ######################################################################
 #
-# Copyright 2000-2004 by Wilson Snyder.  This program is free software;
-# you can redistribute it and/or modify it under the terms of either the GNU
-# General Public License or the Perl Artistic License.
+# This program is Copyright 2002 by Wilson Snyder.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of either the GNU General Public License or the
+# Perl Artistic License, with the exception that it cannot be placed
+# on a CD-ROM or similar media for commercial distribution without the
+# prior approval of the author.
 # 
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 # 
+# If you do not have a copy of the GNU General Public License write to
+# the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
+# MA 02139, USA.
 ######################################################################
 
 package Schedule::Load::Hosts::Host;
@@ -22,22 +29,19 @@ require AutoLoader;
 use Schedule::Load qw(_min _max);
 use Schedule::Load::Hosts::Proc;
 
-use Safe;
-use Carp;
 use strict;
-use vars qw($VERSION $AUTOLOAD $Debug);
+use vars qw($VERSION $AUTOLOAD);
+use Carp;
 
 ######################################################################
 #### Configuration Section
 
 # Other configurable settings.
-$VERSION = '2.022';
+$VERSION = '2.090';
 
 ######################################################################
 #### Globals
 
-$Debug = $Schedule::Load::Debug;
-
 ######################################################################
 #### Special status
 
@@ -87,27 +91,10 @@ sub get_undef {
     }
 }
 
-######################################################################
-#### Matching
-
-sub host_match {
-    my $self = shift; ($self && ref($self)) or croak 'usage: '.__PACKAGE__.'->classes_match(classesref))';
-    my %params = (classes=>[],
-		  match_cb=> undef,
-		  allow_reserved=>1,
-		  @_,
-		  );
-    # For use of Hosts::hosts_match
-    return ($self->classes_match($params{classes})
-	    && $self->eval_match ($params{match_cb})
-	    && (!$self->reserved || $params{allow_reserved})
-	    );
-}
-
 sub classes_match {
-    my $self = shift; ($self && ref($self)) or croak 'usage: '.__PACKAGE__.'->classes_match(classesref))';
+    my $self = shift; ($self && ref($self)) or croak 'usage: '.__PACKAGE__.'->classes_match(field, classesref))';
     my $classesref = shift;
-    return 1 if !defined $classesref || !defined $classesref->[0];  # Null reference means match everything
+    return 1 if !defined $classesref;  # Null reference means match everything
     (ref($classesref)) or croak 'usage: '.__PACKAGE__.'->classes_match(field, classesref))';
     foreach (@{$classesref}) {
 	return 1 if get_undef($self, $_);
@@ -115,52 +102,9 @@ sub classes_match {
     return 0;
 }
 
-sub eval_match {
-    my $self = shift; ($self && ref($self)) or croak 'usage: '.__PACKAGE__.'->eval_match(subroutine)';
-    my $subref = shift;
-    return 1 if !defined $subref;  # Null reference means match everything
-    return $self->_eval_generic_cb($subref);
-}
-
-sub _eval_generic_cb {
-    my $self = shift;
-    my $subref = shift;
-    # Call &$subref($self) in safe container
-    if (ref $subref) {
-	return $subref->($self);
-    } else {
-	my $compartment = new Safe;
-	$compartment->permit(qw(:base_core));
-	$@ = "";
-	@_ = ($self);  # Arguments to pass to reval
-	my $code = $compartment->reval($subref);
-	if ($@ || !$code) {
-	    print "eval_match: $@: $subref\n" if $Debug;
-	    return 0;
-	}
-	my $result = $code->($self);
-	if ($Debug) {   # Try again in non-safe container
-	    @_ = ($self);  # Arguments to pass to reval
-	    my $dcode = eval($subref);
-	    my $dresult = $dcode->($self);
-	    die "%Error: Safe mismatch: '$result' ne '$dresult'\n" if $dresult ne $result;
-	}
-	return $result;
-    }
-}
-
 ######################################################################
 #### Special accessors
 
-sub cpus_slash {
-    my $self = shift;
-    if ($self->cpus != $self->physical_cpus) {
-	return $self->physical_cpus."/".$self->cpus;
-    } else {
-	return $self->cpus;
-    }
-}
-
 sub top_processes {
     my $self = shift; ($self && ref($self)) or croak 'usage: '.__PACKAGE__.'->key(key))';
     my @keys = (values %{$self->{dynamic}{proc}});
@@ -169,26 +113,10 @@ sub top_processes {
     return (wantarray ? @keys : \@keys);
 }
 
-sub holds {
-    my $self = shift;
-    return if !$self->{dynamic}{holds};
-    return (sort {$a->compare_pri_time($b)} (@{$self->{dynamic}{holds}}));
-}
-
-sub free_cpus {
-    my $self = shift;
-    # How many more jobs host can take before we should turn off new jobs
-    my $free = ($self->cpus - $self->adj_load);
-    $free = 0 if ($free < 0);
-    $free = int ($free + .7);
-    return $free;
-}
-
-sub rating_cb {
+sub rating {
     my $self = shift; ($self && ref($self)) or croak 'usage: '.__PACKAGE__.'->key(key))';
     # How fast can we process a single job?
-    # 0 indicates can't load this host
-    # closer to 0 are the best ratings (as 'bad' is open-ended)
+    # 0 is the best rating possible (as 'bad' is open-ended)
     if ($self->get_undef('load_limit')
 	&& $self->load_limit <= $self->adj_load) {
 	# Illegal to load this host more
@@ -205,27 +133,9 @@ sub rating_cb {
     # Discount by cpus & frequency
     $rate /= $self->cpus;
     $rate /= $self->max_clock * 0.4;   # 1 free cpu at 300Mhz beat 50% of a 600 Mhz cpu
-    $rate *= ($self->get_undef('rating_mult') || 1.0);
 
     #printf "%f * (%d+%d+1) / %f / %f = %f\n", ($self->total_pctcpu+1), $self->report_load, $self->adj_load, $self->cpus, $self->max_clock, $rate if $Debug;
-    return 0 if $rate<=0;
-    $rate = log($rate);		# Make a more readable number
-    $rate += ($self->get_undef('rating_adder') || 0);
-    return $rate;
-}
-
-sub rating {
-    my $self = shift; ($self && ref($self)) or croak 'usage: '.__PACKAGE__.'->rating(subroutine)';
-    my $subref = shift;
-    return $self->rating_cb() if !defined $subref;  # Null reference means default callback
-    return $self->_eval_generic_cb($subref);
-}
-
-sub rating_text {
-    my $self = shift; ($self && ref($self)) or croak 'usage: '.__PACKAGE__.'->rating(subroutine)';
-    return "inf" if $self->reserved;
-    return "inf" if !$self->rating;
-    return sprintf("%4.2f", $self->rating);
+    return ($rate>0)?log($rate):0;	# Make a more readable number
 }
 
 ######################################################################
@@ -236,27 +146,22 @@ sub AUTOLOAD {
     my $type = ref($self) or croak "$self is not an ".__PACKAGE__." object";
     
     (my $field = $AUTOLOAD) =~ s/.*://; # Remove package
+    return if $field eq "DESTROY";
   
     if (exists ($self->{dynamic}{$field})) {
-	# Dynamic variables stay dynamic
-	eval "sub $field { return \$_[0]->{dynamic}{$field}; }";
+	eval "sub $field { my \$self=shift; return \$self->{dynamic}{$field}; }";
 	return $self->{dynamic}{$field};
     } elsif (exists ($self->{stored}{$field})) {
-	# Stored variables can move to/from const variables
-	eval "sub $field { return (exists \$_[0]->{stored}{$field} "
-	    ."? \$_[0]->{stored}{$field} : \$_[0]->{const}{$field}); }";
+	eval "sub $field { my \$self=shift; return \$self->{stored}{$field}; }";
 	return $self->{stored}{$field};
     } elsif (exists ($self->{const}{$field})) {
-	eval "sub $field { return (exists \$_[0]->{stored}{$field} "
-	    ."? \$_[0]->{stored}{$field} : \$_[0]->{const}{$field}); }";
+	eval "sub $field { my \$self=shift; return \$self->{const}{$field}; }";
 	return $self->{const}{$field};
     } else {
 	croak "$type->$field: Unknown ".__PACKAGE__." field $field";
     }
 }
 
-sub DESTROY {}
-
 ######################################################################
 ######################################################################
 #### Package return
@@ -282,18 +187,6 @@ host obtained via the Schedule::Load::Host package.
 
 =over 4 
 
-=item classes_match
-
-Passed an array reference.  Returns true if this host's class matches any
-class in the array referenced.
-
-=item eval_match
-
-Passed a subroutine reference that takes a single argument of a host
-reference.  Returns true if the subroutine returns true.  It may also be
-passed a string which forms a subroutine ("sub { my $self = shift; ....}"),
-in which case the string will be evaluated in a safe container.
-
 =item fields
 
 Returns all information fields for this host.
@@ -315,12 +208,12 @@ are described below.
 
 =over 4 
 
-=item adj_load
+=item top_processes
 
-Total number of processes in run or on processor state, adjusted for any
-jobs that have a specific fixed_load or hold time, and adjusted for jobs
-that have not yet scheduled but are collecting resources for a new run.
-This is the load used for picking hosts.
+Returns a reference to a list of top process objects,
+C<Schedule::Load::Hosts::Proc> to access the information for each process.
+In an array context, returns a list; In a a scalar context, returns a
+reference to a list.
 
 =item archname
 
@@ -328,18 +221,7 @@ Architecture name from Perl build.
 
 =item cpus
 
-Number of CPUs.  On hyperthreaded Linux systems, this indicates the maximum
-number of simultaneous threads that may execute; see physical_cpus for the
-real physical CPU count.
-
-=item cpus_slash
-
-Returns a string with the number of cpus, or in hyperthreaded systems, the
-number of physical cpus "/" the number of SMT cpus.
-
-=item holds
-
-Returns list of L<Schedule::Load::Hosts::Hold> objects, sorted by age.
+Number of CPUs.
 
 =item hostname
 
@@ -358,10 +240,6 @@ of CPUs to not allow overloading of a machine.  Undefined if no limit.
 
 Operating system name from Perl build.
 
-=item physical_cpus
-
-Number of CPUs physically present.
-
 =item reservable
 
 If true, this host may be reserved for exclusive use by a user.
@@ -375,46 +253,32 @@ start time comment.
 
 System type from Perl build.
 
-=item top_processes
-
-Returns a reference to a list of top process objects,
-L<Schedule::Load::Hosts::Proc> to access the information for each process.
-In an array context, returns a list; In a a scalar context, returns a
-reference to a list.
-
 =item total_load
 
 Total number of processes in run or on processor state.
 
+=item adj_load
+
+Total number of processes in run or on processor state, adjusted for any
+jobs that have a specific fixed_load or hold time.  This is the load used
+for picking hosts.
+
 =item total_pctcpu
 
 Total CPU percentage used by all processes.
 
-=item total_rss
-
-Total resident memory used by all processes.
-
-=item total_size
-
-Total memory size, resident and swapped, used by all processes. This will
-often exceed the physical memory size.
-
 =back
 
+=head1 SEE ALSO
+
+C<Schedule::Load>, C<Schedule::Load::Hosts>, C<Schedule::Load::Hosts::Proc>
+
 =head1 DISTRIBUTION
 
-The latest version is available from CPAN and from L<http://www.veripool.com/>.
-
-Copyright 1998-2004 by Wilson Snyder.  This package is free software; you
-can redistribute it and/or modify it under the terms of either the GNU
-Lesser General Public License or the Perl Artistic License.
+The latest version is available from CPAN.
 
 =head1 AUTHORS
 
 Wilson Snyder <wsnyder@wsnyder.org>
 
-=head1 SEE ALSO
-
-L<Schedule::Load>, L<Schedule::Load::Hosts>, L<Schedule::Load::Hosts::Proc>
-
 =cut
