@@ -1,16 +1,23 @@
 # Schedule::Load::Hosts::Proc.pm -- Process information
-# $Id: Proc.pm,v 1.29 2005/10/13 12:15:30 wsnyder Exp $
+# $Id: Proc.pm,v 1.14 2002/08/01 14:46:03 wsnyder Exp $
 ######################################################################
 #
-# Copyright 2000-2004 by Wilson Snyder.  This program is free software;
-# you can redistribute it and/or modify it under the terms of either the GNU
-# General Public License or the Perl Artistic License.
+# This program is Copyright 2002 by Wilson Snyder.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of either the GNU General Public License or the
+# Perl Artistic License, with the exception that it cannot be placed
+# on a CD-ROM or similar media for commercial distribution without the
+# prior approval of the author.
 # 
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 # 
+# If you do not have a copy of the GNU General Public License write to
+# the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
+# MA 02139, USA.
 ######################################################################
 
 package Schedule::Load::Hosts::Proc;
@@ -29,7 +36,7 @@ use Carp;
 #### Configuration Section
 
 # Other configurable settings.
-$VERSION = '2.022';
+$VERSION = '2.090';
 
 ######################################################################
 #### Globals
@@ -61,17 +68,13 @@ sub get {
 sub time_hhmm {
     my $self = shift; ($self && ref($self)) or croak 'usage: '.__PACKAGE__.'->get(field))';
     return undef if (!defined $self->{time});
-    return $self->format_hhmm($self->time);
-}
-
-sub format_hhmm {
-    my $self = shift;
-    my $runtime = shift;
+    my $runtime = $self->time;
     if ($runtime >= 2*3600) {
-	return sprintf "%3.1fH", int($runtime/360)/10;
+	$runtime = sprintf "%3.1fH", int($runtime/360)/10;
     } else {
-	return sprintf "%3d:%02d", int($runtime/60), $runtime%60;
+	$runtime = sprintf "%3d:%02d", int($runtime/60), $runtime%60;
     }
+    return $runtime;
 }
 
 ######################################################################
@@ -82,14 +85,15 @@ sub AUTOLOAD {
     my $type = ref($self) or croak "$self is not an ".__PACKAGE__." object";
     
     (my $field = $AUTOLOAD) =~ s/.*://; # Remove package
+    return if $field eq "DESTROY";
+  
     if (exists ($self->{$field})) {
-	eval "sub $field { return \$_[0]->{$field}; }";
+	eval "sub $field { my \$self=shift; return \$self->{$field}; }";
 	return $self->{$field};
     } else {
 	croak "$type->$field: Unknown ".__PACKAGE__." field $field";
     }
 }
-sub DESTROY {}
 
 ######################################################################
 #### Package return
@@ -132,8 +136,8 @@ Returns the value of a specific field for this process.
 =head1 ACCESSORS
 
 A accessor exists for each field returned by the fields() call.  Typical
-elements are described below.  All fields that L<Proc::ProcessTable>
-supports are also accessible.
+elements are described below.  All fields that C<Proc::ProcessTable>
+supports are also included here.
 
 =over 4 
 
@@ -152,20 +156,16 @@ Texual user name running this process.
 
 =back
 
+=head1 SEE ALSO
+
+C<Schedule::Load>, C<Schedule::Load::Hosts>, C<Schedule::Load::Hosts::Host>
+
 =head1 DISTRIBUTION
 
-The latest version is available from CPAN and from L<http://www.veripool.com/>.
-
-Copyright 1998-2004 by Wilson Snyder.  This package is free software; you
-can redistribute it and/or modify it under the terms of either the GNU
-Lesser General Public License or the Perl Artistic License.
+The latest version is available from CPAN.
 
 =head1 AUTHORS
 
 Wilson Snyder <wsnyder@wsnyder.org>
 
-=head1 SEE ALSO
-
-L<Schedule::Load>, L<Schedule::Load::Hosts>, L<Schedule::Load::Hosts::Host>
-
 =cut
