@@ -1,16 +1,23 @@
 # Schedule::Load::Reporter.pm -- distributed lock handler
-# $Id: Reporter.pm,v 1.59 2005/10/13 12:15:30 wsnyder Exp $
+# $Id: Reporter.pm,v 1.32 2002/08/01 15:42:00 wsnyder Exp $
 ######################################################################
 #
-# Copyright 2000-2004 by Wilson Snyder.  This program is free software;
-# you can redistribute it and/or modify it under the terms of either the GNU
-# General Public License or the Perl Artistic License.
+# This program is Copyright 2002 by Wilson Snyder.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of either the GNU General Public License or the
+# Perl Artistic License, with the exception that it cannot be placed
+# on a CD-ROM or similar media for commercial distribution without the
+# prior approval of the author.
 # 
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 # 
+# If you do not have a copy of the GNU General Public License write to
+# the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
+# MA 02139, USA.
 ######################################################################
 
 package Schedule::Load::Reporter;
@@ -27,18 +34,14 @@ use Proc::ProcessTable;
 use Unix::Processors;
 use Storable qw (nstore retrieve);
 use Schedule::Load qw (:_utils);
-use Schedule::Load::FakeReporter;
 use Sys::Hostname;
 use Time::HiRes qw (gettimeofday);
-use IPC::PidStat;
 use Config;
 
 use strict;
-use vars qw($VERSION $RSCHLIB $Debug %User_Names %Pid_Inherit 
+use vars qw($VERSION $Debug %User_Names %Pid_Inherit 
 	    @Pid_Time_Base @Pid_Time $Os_Linux
-	    $Distrust_Pctcpu $Divide_Pctcpu_By_Cpu $MicroTime
-	    $Exister
-	    );
+	    $Distrust_Pctcpu $Divide_Pctcpu_By_Cpu);
 use Carp;
 
 ######################################################################
@@ -47,14 +50,11 @@ use Carp;
 # Other configurable settings.
 $Debug = $Schedule::Load::Debug;
 
-$VERSION = '2.022';
-
-$RSCHLIB = '/usr/local/lib';	# Edited by Makefile
+$VERSION = '2.090';
 
 $Os_Linux = $Config{osname} =~ /linux/i;
 $Distrust_Pctcpu = $Config{osname} !~ /solaris/i;	# Only solaris has instantanous reporting
 $Divide_Pctcpu_By_Cpu = 0;   # Older linuxes may require this
-$MicroTime = ($Config{osname} =~ /linux/i) ? 1 : 1000;  # Fix in Proc::ProcessTable 0.40
 
 ######################################################################
 #### Globals
@@ -84,25 +84,20 @@ sub start {
 	#Undocumented
 	timeout=>$Debug?2:30,		# Sec before host socket connect times out
 	alive_time=>$Debug?10:30,	# Sec to send alive message
-	const_changed=>0,		# const or stored has changed, update in chooser
 	@_};
     bless $self, $class;
 
     # More defaults (can't be above due to needing other elements)
     $self->{const}{hostname} ||= hostname();
     $self->{const}{slreportd_hostname} ||= hostname();
-    $self->{stored_filename} ||= ($RSCHLIB."/rschedule/slreportd_".$self->{const}{hostname}."_store");
+    $self->{stored_filename} ||= ("/usr/local/lib/rschedule/slreportd_".$self->{const}{hostname}."_store");
 
     (defined $self->{dhost}) or croak 'Require a host parameter';
     #foreach (@{$self->{dhost}}) { print "Host $_\n"; }
 
     my $select = IO::Select->new();
 
-    $Exister = new IPC::PidStat();
-    $select->add($Exister->fh);
-
     $self->pt();	# Create process table
-    $self->fake_pt();	# Create process table
 
     # Load constants
     $self->_fill_const;
@@ -121,7 +116,6 @@ sub start {
 		last if ($self->_open_host($host));
 	    }
 	    $select->remove($select->handles);
-	    $select->add($Exister->fh);
 	    $select->add($self->{socket}) if $self->{socket};
 	}
 
@@ -132,42 +126,41 @@ sub start {
       input:
 	foreach my $fh ($select->can_read ($self->{alive_time})) {
 	    print "Servicing input\n" if $Debug;
-	    if ($fh == $Exister->fh) {
-		_exist_traffic();
-	    }
-	    else {
-		# Snarf input
-		if ($inbuffer !~ /\n/) {
-		    my $data='';
-		    my $rv = $fh->sysread($data, POSIX::BUFSIZ);
-		    if (!defined $rv || (length $data == 0)) {
-			next input;
-		    }
-		    $inbuffer .= $data;
-		}
 
-		while ($inbuffer =~ s/(.*?)\n//) {
-		    my $line = $1;
-		    chomp $line;
-		    print "REQ $line\n" if $Debug;
-		    my ($cmd, $params) = _pthaw($line, $Debug);
-		    # Commands
-		    if ($cmd eq "report_get_dynamic") {
-			$self->_fill_and_send;
-		    } elsif ($cmd eq "report_fwd_set") {
-			$self->_set_stored($params);
-		    } elsif ($cmd eq "report_fwd_comment") {
-			$self->_comment($params);
-		    } elsif ($cmd eq "report_fwd_fixed_load") {
-			$self->_fixed_load($params);
-		    } elsif ($cmd eq "report_restart") {
-			# Overall fork loop will deal with it.
-			warn "-Info: report_restart\n" if $Debug;
-			exit(0);
-		    } else {
-			warn "%Error: Bad request from server: $line\n" if $Debug;
-		    }
-		}
+	    # Snarf input
+	    my $data='';
+	    my $rv = $fh->recv($data, POSIX::BUFSIZ, 0);
+	    if (!defined $rv || (length $data == 0)) {
+		$inbuffer = '';
+		next input;
+	    }
+	    $inbuffer .= $data;
+
+	    my $line;
+	    if ($inbuffer =~ s/(.*?)\n//) {
+		$line = $1;
+	    } else {
+		next readchunk;
+	    }
+
+	    chomp $line;
+	    print "REQ $line\n" if $Debug;
+	    my ($cmd, $params) = _pthaw($line, $Debug);
+	    # Commands
+	    if ($cmd eq "report_get_dynamic") {
+		$self->_fill_and_send;
+	    } elsif ($cmd eq "report_fwd_set") {
+		$self->_set_stored($params);
+	    } elsif ($cmd eq "report_fwd_comment") {
+		$self->_comment($params);
+	    } elsif ($cmd eq "report_fwd_fixed_load") {
+		$self->_fixed_load($params);
+	    } elsif ($cmd eq "report_restart") {
+		# Overall fork loop will deal with it.
+		warn "-Info: report_restart\n" if $Debug;
+		exit(0);
+	    } else {
+		warn "%Error: Bad request from server: $line\n" if $Debug;
 	    }
 	}
     }
@@ -185,15 +178,6 @@ sub pt {
     return $self->{pt};
 }
 
-sub fake_pt {
-    my $self = shift;
-    if (!$self->{fake_pt}) {
-	$self->{fake_pt} = Schedule::Load::FakeReporter::ProcessTable
-	    ->new (reportref=>$self);
-    }
-    return $self->{fake_pt};
-}
-
 ######################################################################
 ######################################################################
 #### Sending
@@ -213,11 +197,8 @@ sub _open_host {
     $self->{socket} = undef if (!$fh || !$fh->connected());
     if ($self->{socket}) {
 	# Send constants to the host, that will tell it we live
-	$self->{stored_read} = 0;   # Reread stored info in case redundant reporters
-	$self->{const_changed} = 1;
-	$self->{const}{_update} = 0;
+	$self->_send_hash('const');
 	$self->_fill_and_send;
-	$self->{const}{_update} = 1;   # So chooser can skip calling start function
     }
     print "   Host $host $self->{port} is ".($self->{socket}?"up":"down")."!\n" if $Debug;
     return $self->{socket};
@@ -230,8 +211,8 @@ sub _alive_check {
     my $fh = $self->{socket};
     # Below may die if slchoosed goes down:
     # Our fork() loop will catch it and restart
-    my $ok = $fh->send_and_check($msg);
-    if (!$ok || !$fh || !$fh->connected()) { $self->{socket} = undef; }
+    print $fh $msg;
+    if (!$fh || !$fh->connected()) { $self->{socket} = undef; }
 }
 
 ######################################################################
@@ -244,11 +225,7 @@ sub _fill_and_send {
     # Fill dynamic values and send
     $self->_fill_stored;
     $self->_fill_dynamic;
-    if ($self->{const_changed}) {
-	$self->{const_changed} = 0;
-	$self->_send_hash('const');
-	$self->_send_hash('stored');
-    }
+    $self->_send_hash('stored');
     # Dynamic must be last, it triggers sending info back to user
     $self->_send_hash('dynamic');	
 }
@@ -257,12 +234,10 @@ sub _fill_const {
     my $self = shift;
     # fill constant values into self
     # (Values that don't change with loading -- known at startup)
-    $self->{const_changed} = 1;
  
     # Load our required keys
-    $self->{const}{cpus}          ||= Unix::Processors->max_online();
-    $self->{const}{physical_cpus} ||= Unix::Processors->max_physical();
-    $self->{const}{max_clock}     ||= Unix::Processors->max_clock();
+    $self->{const}{cpus}      ||= Unix::Processors->max_online();
+    $self->{const}{max_clock} ||= Unix::Processors->max_clock();
     $self->{const}{osname}    ||= $Config{osname};
     $self->{const}{osvers}    ||= $Config{osvers};
     $self->{const}{archname}  ||= $Config{archname};
@@ -305,7 +280,7 @@ sub _fill_dynamic_pid {
 	$procref->{nice0} = $procref->{nice} - 20;
     }
 
-    $procref->{time} = $p->time / 1000.0 / (1000.0/$MicroTime);
+    $procref->{time} = $p->time / 1000.0;
 
     my $state = $p->state;
     $state = "cpu".$p->onpro if ($state eq "onprocessor");
@@ -330,8 +305,6 @@ sub _fill_dynamic {
 		        fixed_load => 0,
 			report_load => 0,
 			total_pctcpu => 0,
-			total_size => 0,
-			total_rss => 0,
 		    };
 
     my ($sec, $usec) = gettimeofday();
@@ -340,21 +313,17 @@ sub _fill_dynamic {
     @Pid_Time_Base = ($sec,$usec);
 
     # Note the $p refs cannot be cached, they change when a new table call occurs
-    my @pidlist;
-    if (!$self->{fake}) {
-	push @pidlist, @{$self->pt->table};
-    }
-    push @pidlist, @{$self->fake_pt->table};
+    my $pidlist = $self->pt->table;
 
     my %pidinfo = ();
 
     # Find all parental references (should cache this at some point)
-    foreach my $p (@pidlist) {
+    foreach my $p (@{$pidlist}) {
 	$pidinfo{$p->pid}{parent} = $p->ppid;
     }
 
     # Push all logit's down towards parents
-    foreach my $p (@pidlist) {
+    foreach my $p (@{$pidlist}) {
 	# See which PIDs we will log
 	my $pctcpu = $p->pctcpu || 0;
 	$pctcpu = 0 if ($pctcpu eq "inf");	# Linux
@@ -370,14 +339,14 @@ sub _fill_dynamic {
 		    ,$p->pid, $sec, $p->start, $sec-$p->start, $ustime, $pctcpu
 		    if 0;
 	    } else {
-		$pctcpu = 100*(( ($ustime-$Pid_Time[$p->pid][1]) * $MicroTime)
+		$pctcpu = 100*(( ($ustime-$Pid_Time[$p->pid][1]) * 1000)
 			       / $deltastamp
 			       );
 		$pctcpu /= $self->{const}{cpus} if $Divide_Pctcpu_By_Cpu;
 		printf "PIDCONT %d PCT %s CLOCK %d UTIME %d-%d=%d LOAD %f\n"
 		    ,$p->pid, $p->pctcpu||0, $deltastamp,
 		    ,$ustime, $Pid_Time[$p->pid][1], $ustime-$Pid_Time[$p->pid][1],
-		    ,$pctcpu if 1;
+		    ,$pctcpu if 0;
 	    }
 	    $Pid_Time[$p->pid] = [$p->start, $ustime];
 	}
@@ -398,12 +367,13 @@ sub _fill_dynamic {
 	}
     }
 
-    foreach my $p (@pidlist) {
+    foreach my $p (@{$pidlist}) {
 	my $fixed_load = undef;
 	my $cmndcomment = undef;
 	my $logit = $pidinfo{$p->pid}{logit};
 	if ($p->uid) { # not root
 	    my $searchpid = $p->pid;
+	    my $indent = 0;
 	    while ($searchpid) {
 		if (defined $Pid_Inherit{$searchpid}) {
 		    if ((!defined $fixed_load)
@@ -428,7 +398,6 @@ sub _fill_dynamic {
 	# Load any processes with lots of time, or with fixed_loading
 	# that isn't otherwise accounted for
 	my $pctcpu = $pidinfo{$p->pid}{pctcpu};
-	$pctcpu = 0 if $pctcpu eq 'nan';
 	if ($logit) {
 	    _fill_dynamic_pid ($self, $p, $pctcpu);
 	    $self->{dynamic}{proc}{$p->pid}{cmndcomment} = $cmndcomment if $cmndcomment;
@@ -441,17 +410,12 @@ sub _fill_dynamic {
 	    $self->{dynamic}{total_load} ++;
 	    $self->{dynamic}{report_load} ++ if !defined $fixed_load;
 	}
-
-	# Count memory
-	$self->{dynamic}{total_size} += _fix_overflow($p->size||0);  # Float, so doesn't overflow
-	$self->{dynamic}{total_rss}  += _fix_overflow($p->rss||0);  # Float, so doesn't overflow
     }
 
     # Look for any fixed loads that died
     # Also add up fixed loading across all fixed_loads
     foreach my $pid (keys %Pid_Inherit) {
-	if (!defined $pidinfo{$pid}
-	    && $Pid_Inherit{$pid}{req_hostname} eq hostname()) {  # Not a fake load on a remote host
+	if (!defined $pidinfo{$pid}) {
 	    delete $Pid_Inherit{$pid};
 	} else {
 	    my $fixed_load = $Pid_Inherit{$pid}{fixed_load};
@@ -473,10 +437,7 @@ sub _fixed_load {
     my $pid = $params->{pid};
     print "Fixed load of $load PID $pid\n" if $Debug;
     $Pid_Inherit{$pid}{fixed_load} = $load;
-    $Pid_Inherit{$pid}{pid} = $params->{pid};
     $Pid_Inherit{$pid}{uid} = $params->{uid};
-    $Pid_Inherit{$pid}{req_pid} = $params->{req_pid};
-    $Pid_Inherit{$pid}{req_hostname} = $params->{req_hostname} || $params->{host} || hostname();
     if ($load==0) {
 	delete $Pid_Inherit{$pid};
     }
@@ -489,20 +450,8 @@ sub _comment {
     my $cmndcomment = $params->{comment};
     my $pid = $params->{pid};
     print "Command Commentary '$cmndcomment' PID $pid\n" if $Debug;
-    $Pid_Inherit{$pid}{pid} = $pid;
-    $Pid_Inherit{$pid}{uid} = $params->{uid};
     $Pid_Inherit{$pid}{cmndcomment} = $cmndcomment;
-}
-
-######################################################################
-#### Math
-
-sub _fix_overflow {
-    my $value = shift;
-    # Bug in Proc::ProcessTable before version 0.40 causes 32 bit overflow
-    my $float = 0.1 + $value;
-    $float = 4.0*1024*1024*1024 - $float if $float<0;
-    return $float;
+    $Pid_Inherit{$pid}{uid} = $params->{uid};
 }
 
 ######################################################################
@@ -515,26 +464,8 @@ sub _send_hash {
 
     my $fh = $self->{socket};
     return if !$fh;
-    my $ok = $fh->send_and_check(_pfreeze("report_$field", $self->{$field}, $Debug));
-    if (!$ok || !$fh || !$fh->connected()) { $self->{socket} = undef; }
-}
-
-######################################################################
-######################################################################
-#### Existance
-
-sub _exist_traffic {
-    # Handle UDP responses from our $Exister->pid_request calls.
-    print "UDP PidStat in...\n" if $Debug;
-    my ($pid,$exists,$onhost) = $Exister->recv_stat();
-    return if !defined $pid;
-    return if $exists;   # We only care about known-missing processes
-    print "  UDP PidStat PID $onhost:$pid no longer with us.  RIP.\n" if $Debug;
-    foreach my $pref (values %Pid_Inherit) {
-	if ($pref && $pref->{req_pid}==$pid && $pref->{req_hostname} eq $onhost) {
-	    delete $Pid_Inherit{$pref->{pid}};
-	}
-    }
+    print $fh _pfreeze("report_$field", $self->{$field}, $Debug);
+    if (!$fh || !$fh->connected()) { $self->{socket} = undef; }
 }
 
 ######################################################################
@@ -557,7 +488,6 @@ sub _fill_stored {
 	    print "Retrieve $self->{stored_filename}\n" if $Debug;
 	    $self->{stored} = retrieve($self->{stored_filename});
 	}
-	$self->{const_changed} = 1;
 	$self->{stored_read} = 1;
     }
 }
@@ -568,21 +498,15 @@ sub _set_stored {
     # Set a stored field to a given value
     
     $self->_fill_stored();	# Make sure up-to-date
-    $self->{const_changed} = 1;
 
     foreach my $var (keys %{$params}) {
 	my $value = $params->{$var};
 	next if $var eq "host";
 	print "_set_const($var = $value)\n" if $Debug;
-	if ($params->{set_const}) {
-	    $self->{const}{$var} = $value;
-	} else {
-	    $self->{stored}{$var} = $value;
-	}
+	$self->{stored}{$var} = $value;
     }
     
-    if (!$params->{set_const}
-	&& defined $self->{stored_filename}) {
+    if (defined $self->{stored_filename}) {
 	print "Store $self->{stored_filename}\n" if $Debug;
 	nstore $self->{stored}, $self->{stored_filename};
 	chmod 0666, $self->{stored_filename};
@@ -598,6 +522,7 @@ __END__
 
 =pod
 
+
 =head1 NAME
 
 Schedule::Load::Reporter - Distributed load reporting daemon
@@ -611,7 +536,7 @@ Schedule::Load::Reporter - Distributed load reporting daemon
 
 =head1 DESCRIPTION
 
-L<Schedule::Load::Reporter> on startup connects to the requested server
+C<Schedule::Load::Reporter> on startup connects to the requested server
 host and port.  The server connected to can then poll this host for
 information about system configuration and current loading conditions.
 
@@ -637,11 +562,6 @@ host is only used if the first is down, and so on down the list.
 The port number of slchoosed.  Defaults to 'slchoosed' looked up via
 /etc/services, else 1752.
 
-=item fake
-
-Specifies load management should not be used, for reporting of a "fake"
-hosts' status or scheduling a non-host related resource, like a license.
-
 =item min_pctcpu
 
 The minimum percentage of the CPU that a job must have to be included in
@@ -650,28 +570,24 @@ the list of top processes sent to the client.  Defaults to 3.  Setting to
 
 =item stored_filename
 
-The filename to store persistent items in, such as if this host is
+The filename to store persistant items in, such as if this host is
 reserved.  Must be either local-per-machine, or have the hostname in it.
 Defaults to /usr/local/lib/rschedule/slreportd_{hostname}_store.  Set to
-undef to disable persistence (thus if the machine reboots the reservation
-is lost.)   The path must be **ABSOLUTE** as the daemons do a chdir.
+undef to disable persistance (thus if the machine reboots the reservation
+is lost.)   The path must be **ABSOLUTE** as the deamons do a chdir.
 
 =back
 
+=head1 SEE ALSO
+
+C<Schedule::Load>, C<slreportd>
+
 =head1 DISTRIBUTION
 
-The latest version is available from CPAN and from L<http://www.veripool.com/>.
-
-Copyright 1998-2004 by Wilson Snyder.  This package is free software; you
-can redistribute it and/or modify it under the terms of either the GNU
-Lesser General Public License or the Perl Artistic License.
+This package is distributed via CPAN.
 
 =head1 AUTHORS
 
 Wilson Snyder <wsnyder@wsnyder.org>
 
-=head1 SEE ALSO
-
-L<Schedule::Load>, L<slreportd>
-
 =cut
