@@ -1,16 +1,23 @@
 # Schedule::Load::Schedule.pm -- Schedule jobs across a network
-# $Id: Schedule.pm,v 1.46 2005/10/13 12:15:30 wsnyder Exp $
+# $Id: Schedule.pm,v 1.22 2002/08/01 14:46:03 wsnyder Exp $
 ######################################################################
 #
-# Copyright 2000-2004 by Wilson Snyder.  This program is free software;
-# you can redistribute it and/or modify it under the terms of either the GNU
-# General Public License or the Perl Artistic License.
+# This program is Copyright 2002 by Wilson Snyder.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of either the GNU General Public License or the
+# Perl Artistic License, with the exception that it cannot be placed
+# on a CD-ROM or similar media for commercial distribution without the
+# prior approval of the author.
 # 
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 # 
+# If you do not have a copy of the GNU General Public License write to
+# the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
+# MA 02139, USA.
 ######################################################################
 
 package Schedule::Load::Schedule;
@@ -20,7 +27,6 @@ require Exporter;
 
 use Schedule::Load qw (:_utils);
 use Schedule::Load::Hosts;
-use Schedule::Load::ResourceReq;
 use Sys::Hostname;
 use Time::localtime;
 
@@ -33,7 +39,7 @@ use Carp;
 
 # Other configurable settings.
 $Debug = $Schedule::Load::Debug;
-$VERSION = '2.022';
+$VERSION = '2.090';
 @MoY = ('Jan','Feb','Mar','Apr','May','Jun',
 	'Jul','Aug','Sep','Oct','Nov','Dec');
 
@@ -47,7 +53,10 @@ sub new {
     @_ >= 1 or croak 'usage: '.__PACKAGE__.'->new ({options})';
     my $proto = shift;
     return $proto->SUPER::new
-	( scheduled_hosts => [],
+	( night_hours_cb => \&night_hours_p,
+	  favor_host => hostname(),
+	  hold_load => 1,
+	  hold_time => 60,	# secs
 	  @_);
 }
 
@@ -57,23 +66,15 @@ sub new {
 ######################################################################
 #### Accessors
 
-sub scheduled_hosts {
-    my $self = shift; ($self && ref($self)) or croak 'usage: $self->scheduled_hosts (perhaps you forgot to check schedule return for undef)';
-    return (wantarray ? @{$self->{scheduled_hosts}} : $self->{scheduled_hosts});
-}
-
-sub scheduled_hostnames {
-    my $self = shift; ($self && ref($self)) or croak 'usage: $self->scheduled_hosts (perhaps you forgot to check schedule return for undef)';
-    return () if !$self->{scheduled_hosts}[0];
-    my @names = map {$_->hostname; } $self->scheduled_hosts;
-    return @names;
-}
-
 sub hosts_of_class {
     my $self = shift; ($self && ref($self)) or croak 'usage: $self->hosts()';
-    # DEPRECIATED.  Return all hosts matching given class
-    # allow_reserved was ignored in the old implementation...
-    return $self->hosts_match (@_, allow_reserved=>1);
+    my $schparams = $self->_scheduler_params (@_);
+    # Return all hosts matching given class
+    my @keys = ();
+    foreach (@{$self->hosts}) {
+	push @keys, $_ if $_->classes_match ($schparams->{classes});
+    }
+    return (wantarray ? @keys : \@keys);
 }
 
 ######################################################################
@@ -107,7 +108,6 @@ sub reserve {
 		      reserved=>$params->{comment},);
     $self->fetch();
     $host = $self->get_host($params->{host});	# Fetch might have new reference
-    ($host) or croak "%Error: Host $params->{host} not responding";
     ($host->reserved) or croak "%Error: Host $params->{host} didn't accept reservation";
 }
 
@@ -134,7 +134,6 @@ sub release {
 		      reserved=>0,);
     $self->fetch();
     $host = $self->get_host($params->{host});	# Fetch might have new reference
-    ($host) or croak "%Error: Host $params->{host} not responding";
     (!$host->reserved) or croak "%Error: Host $params->{host} didn't accept release";
 }
 
@@ -145,10 +144,8 @@ sub fixed_load {
 	load=>1,
 	uid=>$<,
 	pid=>$$,
-	req_hostname=>hostname(),  # Where to do a pid_exists
-	#req_pid=>pid,
 	@_,};
-    $params->{req_pid} ||= $params->{pid};
+
     print __PACKAGE__."::fixed_load($params->{load})\n" if $Debug;
     $self->_request(_pfreeze( 'report_fwd_fixed_load', $params, $Debug));
 }
@@ -168,85 +165,67 @@ sub hold_release {
 #### Scheduling
 
 sub best {
-    my $self = shift; ($self && ref($self)) or croak 'usage: $self->best';
-    my %params = (allow_none=>0,
-		  @_);
-    print __PACKAGE__."::best()\n" if $Debug;
-    # Backward compatible best function, scheduling on one host without
-    # need to understand new Hold and ResourceReq structures.
+    my $self = shift; ($self && ref($self)) or croak 'usage: $self->best)';
+    my $schparams = $self->_scheduler_params (@_);
 
-    # Make a hold element with passed params
-    my $hold = Schedule::Load::Hold->new(hold_key=>"best",);
-    $hold->set_fields (%{$self},%params);
-    # Make resource requests with passed params
-    my $req = Schedule::Load::ResourceReq->new();
-    $req->set_fields (%{$self},%params);
-    my $rtn = $self->schedule
-	(resources=>[$req],
-	 hold=>$hold,
-	 allow_none=>1,
-	 %params);
-    return undef if !$rtn;
-    return undef if !$rtn || !$rtn->scheduled_hosts;
-    my @hn = $rtn->scheduled_hostnames;
-    return $hn[0];
+    print __PACKAGE__."::best()\n" if $Debug;
+    $self->_schedule_and_get ($schparams);
+    return ($self->{_best}{best});
 }
 
 sub jobs {
-    my $self = shift; ($self && ref($self)) or croak 'usage: $self->jobs';
-    #** Old depreciated interface
+    my $self = shift; ($self && ref($self)) or croak 'usage: $self->jobs)';
+    my $schparams = $self->_scheduler_params (@_);
+
     print __PACKAGE__."::jobs()\n" if $Debug;
-    my @names = $self->idle_host_names(@_);
-    return ($#names+1);
+    $self->_schedule_and_get ($schparams);
+    return ($self->{_best}{jobs});
 }
 
-sub schedule {
+sub _schedule_and_get {
     my $self = shift;
-    my %params = (allow_none=>0,
-		  hold=>undef,	# Schedule::Load::Hold reference, undef not to hold
-		  resources=>[],# Schedule::Load::ResourceReq reference
-		  @_);
+    my $schparams = shift;
 
-    $self->{scheduled_hosts} = [];
-    $self->{_schrtn} = undef;
-    $params{resources}[0] or croak "%Error: Not passed any resources=>[] to schedule,";
-
-    use Data::Dumper; print "SCHEDULE: ",Dumper(\%params) if $Debug;
-    $self->_request(_pfreeze ("schedule", \%params, 0&&$Debug));
-
-    use Data::Dumper; print "RETURN: ",Dumper($self->{_schrtn}) if $Debug;
-    (defined $self->{_schrtn}) or die "%Error: Didn't get proper schedule response\n";
-
-    if (!$self->{_schrtn}{best}) {
-	return undef;
-    } else {
-	# Remap the hostnames to references (can't pass refs across a socket!)
-	foreach my $hostname (@{$self->{_schrtn}{best}}) {
-	    my $host = $self->get_host($hostname);
-	    if (!$host) {
-		# It's a host that wasn't in our cache....
-		print " Gethost $hostname failed, retrying caching\n" if $Debug;
-		$self->kill_cache;
-		$self->fetch;
-		$host = $self->get_host($hostname);
-		if (!$host) {
-		    print " Gethost $hostname retry failed\n" if $Debug;
-		    return undef;  # Next scheduler attempt should make sense of it all...
-		}
-	    }
-	    push @{$self->{scheduled_hosts}}, $host;
-	}
-    }
-    return $self;
+    $self->{_best} = {};
+    $self->_request(_pfreeze ("schedule", $schparams, 0&&$Debug));
+    (defined $self->{_best}{jobs}) or die "%Error: Didn't get proper schedule response\n";
 }
 
 sub night_hours_p {
     # Return true if working hours
-    my $working = ((localtime->hour >= 7 && localtime->hour < 22)
-		   && (localtime->wday >= 1 && localtime->wday < 6)); # M-F
+    my $working = ((localtime->hour >= 6 && localtime->hour < 22)
+		   && (localtime->wday >= 1 && localtime->wday < 6));
     return !$working;
 }
 
+sub _scheduler_params {
+    my $self = shift;
+
+    my $is_night = (&{$self->{night_hours_cb}} ($self));
+    my $schparams = { classes=>[],
+		      allow_none=>0,
+		      favor_host=>$self->{favor_host},
+		      hold_time=> $self->{hold_time},
+		      hold_key=>  undef,
+		      hold_load=> 1,
+		      max_jobs=>  ($is_night ? -1  : -0.2 ),
+		      @_ };
+    # Take a ref to list of classes and add class_ and any night time options
+    # Return ref to hash with scheduler options: classes and is_night
+
+    my @classes = ();
+    foreach (@{$schparams->{classes}}) {
+	$_ = "class_$_" if $_ !~ /^class_/;
+	push @classes, $_;
+	push @classes, $_ . "_night" if $is_night && ($_ !~ /_night$/);
+    }
+    $schparams->{classes} = \@classes;
+    print "schparams=", Data::Dumper::Dumper ($schparams) if $Debug;
+    return $schparams;
+}
+
+#    $self->_request("get const load proc\n");
+
 ######################################################################
 ######################################################################
 #### Changing persistant store's on a host
@@ -255,7 +234,6 @@ sub set_stored {
     my $self = shift; ($self && ref($self)) or croak 'usage: $self->hosts)';
     my $params = {
 	host=>undef,
-	#set_const=>undef,	# If true, put into constant rather then stored data
 	@_,};
 
     print __PACKAGE__."::set_stored($params->{host})\n" if $Debug;
@@ -284,6 +262,7 @@ __END__
 
 =pod
 
+
 =head1 NAME
 
 Schedule::Load::Schedule - Functions for choosing a host among many
@@ -297,7 +276,7 @@ Schedule::Load::Schedule - Functions for choosing a host among many
 
 =head1 DESCRIPTION
 
-This package will allow the most lightly loaded host to be chosen for new
+This package will allow the most lightly loaded host to be choosen for new
 jobs across many machines across a entire network.
 
 It is also a superclass of Schedule::Load::Hosts, so any functions that
@@ -309,19 +288,14 @@ work for that module also work here.
 
 =item best (...)
 
-Returns the hostname of the best host in the network for a single new job.
-Parameters may be parameters specified in this class, Schedule::Load::Hold,
-or Schedule::Load::ResourceReq.  Those packages must be used individually
-if multiple resources need to be scheduled simultaneously.
+Returns the hostname of the best host in the network for a new job.
 
-=item fixed_load (load=>load_value, [pid=>$$], [host=>localhost], [req_pid=>$$, req_hostname=>localhost])
+=item fixed_load (load=>load_value, [pid=>$$], [host=>localhost])
 
 Sets the current process and all children as always having at least the
-load value specified.  This prevents under-counting CPU utilization when a
+load value specified.  This prevents undercounting CPU utilization when a
 large batch job is running which is just paused in the short term to do
-disk IO or sleep.  Requests to fake reporters (resources not associated
-with a CPU) may specify req_pid and req_hostname which are the PID and
-hostname that must continue to exist for the fixed_load to remain in place.
+disk IO or sleep.
 
 =item hold_release (hold_key=>key)
 
@@ -329,7 +303,8 @@ Releases the temporary hold placed with the best function.
 
 =item hosts_of_class (class=>name)
 
-Depreciated, and to be removed in later releases.  Use hosts_match instead.
+Returns C<Schedule::Load::Hosts::Host> objects for every host that matches
+the given class.
 
 =item jobs (...)
 
@@ -346,61 +321,76 @@ release does not have to be the same user that reserved the host.
 =item reserve (host=>hostname, [comment=>comment])
 
 Reserves the machine for exclusive use of the current user.  The host
-chosen must have the reservable flag set.  C<rschedule hosts> will show
+choosen must have the reservable flag set.  C<rschedule hosts> will show
 the host as reserved, along with the provided comment.
 
-=item schedule (hold=>Schedule::Load::Hold ref, resources=>[], [allow_none=>1])
-
-Schedules the passed list of Schedule::Load::ResourceReq resources, and
-holds them using the passed hold key.  If allow_none is set and the loading
-is too high, does not schedule any resources.  Returns a object reference
-to use with scheduled_hosts, or undef if no resources available.
-
-=item scheduled_hosts
-
-Returns a list of Schedule::Load::Host objects that were scheduled using
-the last schedule() call.
-
-=item set_stored (host=>hostname, [set_const=>1], [key=>value])
-
-Set a key/value parameter on the persistent storage on the remote server,
-such as if a class is allowed on that host.  With const=>1, don't make it
-persist, but make it look like the daemon was started with that option;
-when the daemon restarts the information will be lost.
-
 =back
 
 =head1 PARAMETERS
 
 Parameters for the new and fetch calls are shown in
-L<Schedule::Load::Hosts>.
+C<Schedule::Load::Hosts>.
 
 =over 4
 
 =item allow_none
 
 If allow_none is true, if there is less then a free CPU across the entire
-network, then no cpu will be chosen.  This is useful for programs that can
+network, then no cpu will be choosen.  This is useful for programs that can
 dynamically adjust their outstanding job count.  (Presumably you would only
 set allow_none if you already have one job running, or you can get
-live-locked out of getting anything!)
+livelocked out of getting anything!)
+
+=item classes
+
+An array reference of which classes the host must support to allow this job
+to be run on that host.  Defaults to [], which allows any host.
+
+=item favor_host
+
+The hostname to try and choose if all is equal, under the presumption that
+there are disk access time benefits to doing so.  Defaults to the current host.
+
+=item hold_key
+
+A hold key will reserve a job slot on the choosen CPU until a release_hold
+function is called.  This prevents overscheduling a host due to the delay
+between choosing a host with a light load and starting the job on it which
+rases the CPU load of that choosen host.
+
+=item hold_time
+
+Number of seconds to allow the hold to remain before being removed
+automatically.
+
+=item hold_load
+
+Number of cpu loads the hold_key should reserve, defaults to one.
+
+=item max_jobs
+
+Maximum number of jobs that jobs() can return.  Negative fraction indicates
+that percentage of the clump, for example -0.5 will use 50% of free CPUs.
+Defaults to 20% of the free clump during the day, 100% at night.
+
+=item night_hours_cb
+
+Reference to Function for determining if this is night time, defaults to
+M-F 6am-10pm.  When it is nighttime hours, every class passed to the best
+option has a new class with _night appended.
 
 =back
 
+=head1 SEE ALSO
+
+C<Schedule::Load>, C<Schedule::Load::Hosts>, C<rschedule>
+
 =head1 DISTRIBUTION
 
-The latest version is available from CPAN and from L<http://www.veripool.com/>.
-
-Copyright 1998-2004 by Wilson Snyder.  This package is free software; you
-can redistribute it and/or modify it under the terms of either the GNU
-Lesser General Public License or the Perl Artistic License.
+The latest version is available from CPAN.
 
 =head1 AUTHORS
 
 Wilson Snyder <wsnyder@wsnyder.org>
 
-=head1 SEE ALSO
-
-L<Schedule::Load>, L<Schedule::Load::Hosts>, L<rschedule>
-
 =cut
